#
# spec file for package kvm_stat
#
# Copyright (c) 2022 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


Name:           kvm_stat
%define version %(rpm -q --qf '%%{VERSION}' kernel-source)
Version:        %{version}
Release:        22.51
Summary:        Monitoring Tool for KVM guests
License:        GPL-2.0-only
Group:          System/Monitoring
URL:            http://www.kernel.org/
BuildArch:      noarch
BuildRequires:  asciidoc
BuildRequires:  kernel-source >= 5.2.0
BuildRequires:  libxslt-tools

Requires:       python3-curses
Recommends:     logrotate

Recommends:     kernel >= 5.2.0
Conflicts:      qemu < 2.6.90
Conflicts:      qemu-kvm < 2.6.90
BuildRoot:      %{_tmppath}/%{name}-%{version}-build

# Patches 01 to 07 are for jsc#SLE-13784
Source1:        logrotate.kvm_stat
Patch01:        rework-command-line-sequence.patch
Patch02:        switch-to-argparse.patch
Patch03:        add-command-line-switch-s-to-update.patch
Patch04:        add-command-line-switch-c-to-csv.patch
Patch05:        add-command-line-switch-z-skip-zero-records.patch
Patch06:        add-command-line-switch-L-to-log-file.patch
Patch07:        add-sample-systemd-unit.patch
Patch08:        add-restart-delay.patch
# PAtch 09 is for bsc#1202924
Patch09:        tools-kvm_stat-fix-attack-vector-with-user-controlle.patch

%define XXX This package provides a userspace tool "kvm_stat", which displays KVM vm exit \
information as a means of monitoring vm behavior. The data is taken from the\
KVM debugfs files or the vm tracepoints and outputs them as a curses ui or\
simple text.

%description
%{XXX}

%package rebuild
Summary:        Empty package to ensure rebuilding kvm_stat in OBS
Group:          System/Monitoring
%requires_eq kernel-source

%description rebuild
This is empty package that ensures kvm_stat is rebuilt every time
kernel-default is rebuilt in OBS.

There is no reason to install this package.

%prep
# copy necessary files from kernel-source
(tar -C /usr/src/linux -c COPYING tools scripts) | tar -x

# Patches present upstream, since 5.7
%if "%(echo `echo -e "%{version}\\n5.7.0" | sort -V | head -n1 2> /dev/null`)" != "5.7.0"
%patch01 -p1
%patch02 -p1
%patch03 -p1
%patch04 -p1
%endif
# Patches present upstream, since 5.8
%if "%(echo `echo -e "%{version}\\n5.8.0" | sort -V | head -n1 2> /dev/null`)" != "5.8.0"
%patch05 -p1
%patch06 -p1
%patch07 -p1
%patch08 -p1
%endif
%if %{pkg_vcmp kernel-source < 6.1}
%patch09 -p1
%endif

%build
make -C tools/kvm/kvm_stat %{?_smp_mflags}

%install
# OBS checks don't like /usr/bin/env in script interpreter lines
sed -re '1 { s_^#! */usr/bin/env +/_#!/_ ; s_^#! */usr/bin/env +([^/])_#!/usr/bin/\1_ }' -i "tools/kvm/kvm_stat/kvm_stat"
make -C tools kvm_stat_install INSTALL_ROOT=%{buildroot}
%if 0%{?suse_version} > 1500
mkdir -p %{buildroot}%{_distconfdir}/logrotate.d
install -D -m 644 %{SOURCE1} %{buildroot}%{_distconfdir}/logrotate.d/kvm_stat
%else
install -D -m 644 %{SOURCE1} %{buildroot}%{_sysconfdir}/logrotate.d/kvm_stat
%endif
install -D -m 644 tools/kvm/kvm_stat/kvm_stat.service %{buildroot}%{_unitdir}/kvm_stat.service
install -d %{buildroot}%{_sbindir}
ln -sf service %{buildroot}%{_sbindir}/rckvm_stat

%pre
%service_add_pre kvm_stat.service
%if 0%{?suse_version} > 1500
# Prepare for migration to /usr/etc; save any old .rpmsave
for i in logrotate.d/kvm_stat ; do
   test -f %{_sysconfdir}/${i}.rpmsave && mv -v %{_sysconfdir}/${i}.rpmsave %{_sysconfdir}/${i}.rpmsave.old ||:
done
%endif

%if 0%{?suse_version} > 1500
%posttrans
# Migration to /usr/etc, restore just created .rpmsave
for i in logrotate.d/kvm_stat ; do
   test -f %{_sysconfdir}/${i}.rpmsave && mv -v %{_sysconfdir}/${i}.rpmsave %{_sysconfdir}/${i} ||:
done
%endif

%post
%service_add_post kvm_stat.service

%preun
%service_del_preun kvm_stat.service

%postun
%service_del_postun kvm_stat.service

%files
%defattr(-, root, root)
%license COPYING
%{_unitdir}/kvm_stat.service
%if 0%{?suse_version} > 1500
%{_distconfdir}/logrotate.d/kvm_stat
%else
%config(noreplace) %{_sysconfdir}/logrotate.d/kvm_stat
%endif
%{_sbindir}/rckvm_stat
%{_bindir}/kvm_stat
%{_mandir}/man1/kvm_stat*

%files rebuild
%license COPYING

%changelog
* Fri Dec 30 2022 Dominique Leuenberger <dimstar@opensuse.org>
- Apply tools-kvm_stat-fix-attack-vector-with-user-controlle.patch
  only on kernel < 6.1. Starting with 6.1, the fix is included.
* Fri Oct 28 2022 Dario Faggioli <dfaggioli@suse.com>
- Fix security issue about debugfs described in bsc#1202924
  * Added patches:
    tools-kvm_stat-fix-attack-vector-with-user-controlle.patch
* Wed Aug 31 2022 Stefan Schubert <schubi@intern>
- Migration to /usr/etc: Saving user changed configuration files
  in /etc and restoring them while an RPM update.
* Tue Jun 21 2022 Stefan Schubert <schubi@suse.com>
- Moved logrotate files from user specific directory /etc/logrotate.d
  to vendor specific directory /usr/etc/logrotate.d.
* Wed Jul  7 2021 Al Cho <acho@suse.com>
- If the kvm service is enabled and the system rebooted,
  Systemd's initial attempt to start the kvm unit file may
  fail in case the kvm module is not loaded.
  Add an appropriate delay in the unit file. (bsc#1185945)
  * Add patches:
  * add-restart-delay.patch
* Thu Jan 28 2021 Dominique Leuenberger <dimstar@opensuse.org>
- Add a dummy -rebuild package: give OBS/Tumbleweed a hint to tell
  when this package needs a rebuild (boo#1178493).
* Thu Jan 28 2021 Dario Faggioli <dfaggioli@suse.com>
- Install the systemd unit file in the proper place, so that the
  service can be started right away, if the user wants that, without
  having to copy the file in the units directory.
- Integrate with logrotate, by providing a  logrotate config file, as
  the logs can easily become very big.
* Fri Jan  8 2021 Dario Faggioli <dfaggioli@suse.com>
- Get rid of the stale patch file.
* Fri Jan  8 2021 Dario Faggioli <dfaggioli@suse.com>
- Remove broken patch tools-kvm_stat-explicitly-reference-python3.patch
  * the patch does not apply any longer, at least on top of
    Kernel source packages does the same (i.e., replacing /usr/bin/env)
    in their own kernel-source.spec;
  * therefore, let's take care of that directly inside of our own spec
    file instead, which is a solution that always work, no matter what
    has been done in the Kernel source package.
* Wed Dec 23 2020 Dario Faggioli <dfaggioli@suse.com>
- Fix kernel version comparison for selectively applying patches
  * so that it won't break when, e.g., 5.10.0 hits Factory
* Fri Dec  4 2020 Dario Faggioli <dfaggioli@suse.com>
- Include the example service file in the package
  * install it in /usr/share/kvm_stat
  * this deals properly with bsc#1179469
* Fri Nov 13 2020 Dario Faggioli <dfaggioli@suse.com>
- Implement jsc#SLE-13784
  * Add patches:
  * rework-command-line-sequence.patch
  * switch-to-argparse.patch
  * add-command-line-switch-s-to-update.patch
  * add-command-line-switch-c-to-csv.patch
  * add-command-line-switch-z-skip-zero-records.patch
  * add-command-line-switch-L-to-log-file.patch
  * add-sample-systemd-unit.patch
  * patching is conditional, depending on kernel version
* Wed Mar 11 2020 Bruce Rogers <brogers@suse.com>
- Remove legacy compatability patching, since that usage no longer
  needs to be single sourced with this current package. Python3 is
  now assumed. Also get rid of the final patch based on running
  2to3, since those changes are not really needed: the kvm_stat now
  in the current kernel source is good python3 code
  Patches removed:
  0001-tools-kvm_stat-fix-event-counts-display-for-interrup.patch
  0002-tools-kvm_stat-fix-undue-use-of-initial-sleeptime.patch
  0003-tools-kvm_stat-remove-unnecessary-header-redraws.patch
  0004-tools-kvm_stat-simplify-line-print-logic.patch
  0005-tools-kvm_stat-remove-extra-statement.patch
  0006-tools-kvm_stat-simplify-initializers.patch
  0007-tools-kvm_stat-move-functions-to-corresponding-class.patch
  0008-tools-kvm_stat-show-cursor-in-selection-screens.patch
  0009-tools-kvm_stat-display-message-indicating-lack-of-ev.patch
  0010-tools-kvm_stat-make-heading-look-a-bit-more-like-top.patch
  0011-tools-kvm_stat-rename-Current-column-to-CurAvg-s.patch
  0012-tools-kvm_stat-add-new-interactive-command-h.patch
  0013-tools-kvm_stat-add-new-interactive-command-s.patch
  0014-tools-kvm_stat-add-new-interactive-command-o.patch
  0015-tools-kvm_stat-display-guest-list-in-pid-guest-selec.patch
  0016-tools-kvm_stat-fix-error-on-interactive-command-g.patch
  0017-tools-kvm_stat-add-new-command-line-switch-i.patch
  0018-tools-kvm_stat-add-new-interactive-command-b.patch
  0019-tools-kvm_stat-add-f-help-to-get-the-available-event.patch
  0020-tools-kvm_stat-Add-Python-3-support-to-kvm_stat.patch
  0021-tools-kvm_stat-fix-command-line-option-g.patch
  0022-tools-kvm_stat-fix-drilldown-in-events-by-guests-mod.patch
  0023-tools-kvm_stat-fix-missing-field-update-after-filter.patch
  0024-tools-kvm_stat-fix-extra-handling-of-help-with-field.patch
  0025-tools-kvm_stat-add-hint-on-f-help-to-man-page.patch
  0026-tools-kvm_stat-fix-child-trace-events-accounting.patch
  0027-tools-kvm_stat-handle-invalid-regular-expressions.patch
  0028-tools-kvm_stat-suppress-usage-information-on-command.patch
  0029-tools-kvm_stat-stop-ignoring-unhandled-arguments.patch
  0030-tools-kvm_stat-add-line-for-totals.patch
  0031-tools-kvm_stat-sort-f-help-output.patch
  0032-tools-kvm_stat-simplify-the-sortkey-function.patch
  0033-tools-kvm_stat-use-a-namedtuple-for-storing-the-valu.patch
  0034-tools-kvm_stat-use-a-more-pythonic-way-to-iterate-ov.patch
  0035-tools-kvm_stat-avoid-is-for-equality-checks.patch
  0036-tools-kvm_stat-fix-crash-when-filtering-out-all-non-.patch
  0037-tools-kvm_stat-print-error-on-invalid-regex.patch
  0038-tools-kvm_stat-fix-debugfs-handling.patch
  0039-tools-kvm_stat-mark-private-methods-as-such.patch
  0040-tools-kvm_stat-eliminate-extra-guest-pid-selection-d.patch
  0041-tools-kvm_stat-separate-drilldown-and-fields-filteri.patch
  0042-tools-kvm_stat-group-child-events-indented-after-par.patch
  0043-tools-kvm_stat-print-Total-line-for-multiple-events-.patch
  0044-tools-kvm_stat-Don-t-use-deprecated-file.patch
  0045-tools-kvm_stat-fix-python3-issues.patch
  0046-tools-kvm_stat-apply-python-2to3-fixes-to-kvm_stat.patch
  0047-tools-kvm_stat-apply-python-2to3-fixes-to-kvm_stat2.patch
  0048-tools-kvm_stat-apply-python-2to3-fixes-to-kvm_stat3.patch
  0049-tools-kvm_stat-switch-python-reference-to-be-explici.patch
  0050-tools-kvm_stat-switch-python-reference-again.patch
  Patches added:
  tools-kvm_stat-explicitly-reference-python3.patch
* Tue Sep 24 2019 Bruce Rogers <brogers@suse.com>
- Fix our patching for Leap 15.1 (boo#1140899)
* Thu Sep 12 2019 Bruce Rogers <brogers@suse.com>
- Detect SLE15-SP2 / Leap 15.2 and patch accordingly
* Tue Aug  6 2019 Bruce Rogers <brogers@suse.com>
- Adjust a patch that no longer applies due to upstream (non-code)
  changes
* Thu Mar 14 2019 Bruce Rogers <brogers@suse.com>
- Add python3 tweak to be compatible with v5.0 kernel source
  (bsc#1116822)
  + 0050-tools-kvm_stat-switch-python-reference-again.patch
* Thu Nov 29 2018 Larry Dewey <ldewey@suse.com>
  * Changed the spec file to build SLE15-SP1 with the s390x patches,
  and added logic to the spec file to uniquely build Factory and
  SLE-15 as they do not require the first 44 patches, and the code
  base is not the same
- Making kvm_stat use python3 for openSUSE (bsc#1116822)
  - 0047-tools-kvm_stat-switch-python-reference-to-be-explici.patch
  + 0047-tools-kvm_stat-apply-python-2to3-fixes-to-kvm_stat2.patch
  + 0048-tools-kvm_stat-apply-python-2to3-fixes-to-kvm_stat3.patch
  + 0049-tools-kvm_stat-switch-python-reference-to-be-explici.patch
* Mon Nov 26 2018 Bruce Rogers <brogers@suse.com>
- Make kvm_stat use python3 (bsc#1116822)
  + 0044-tools-kvm_stat-Don-t-use-deprecated-file.patch
  + 0045-tools-kvm_stat-fix-python3-issues.patch
  + 0046-tools-kvm_stat-apply-python-2to3-fixes-to-kvm_stat.patch
  + 0047-tools-kvm_stat-switch-python-reference-to-be-explici.patch
* Mon Nov 19 2018 Larry Dewey <ldewey@suse.com>
  * Incorporating patches and comments produced by Fei Li for
  SLE15-SP1
- Backport 43 kvm_stat patches between kernel v4.16-* and current
  v4.12.14 for the kvm_stat package. Actually some of the patches
  are s390x specific, and some are noarch. (FATE#325017)
  + 0001-tools-kvm_stat-fix-event-counts-display-for-interrup.patch
  + 0002-tools-kvm_stat-fix-undue-use-of-initial-sleeptime.patch
  + 0003-tools-kvm_stat-remove-unnecessary-header-redraws.patch
  + 0004-tools-kvm_stat-simplify-line-print-logic.patch
  + 0005-tools-kvm_stat-remove-extra-statement.patch
  + 0006-tools-kvm_stat-simplify-initializers.patch
  + 0007-tools-kvm_stat-move-functions-to-corresponding-class.patch
  + 0008-tools-kvm_stat-show-cursor-in-selection-screens.patch
  + 0009-tools-kvm_stat-display-message-indicating-lack-of-ev.patch
  + 0010-tools-kvm_stat-make-heading-look-a-bit-more-like-top.patch
  + 0011-tools-kvm_stat-rename-Current-column-to-CurAvg-s.patch
  + 0012-tools-kvm_stat-add-new-interactive-command-h.patch
  + 0013-tools-kvm_stat-add-new-interactive-command-s.patch
  + 0014-tools-kvm_stat-add-new-interactive-command-o.patch
  + 0015-tools-kvm_stat-display-guest-list-in-pid-guest-selec.patch
  + 0016-tools-kvm_stat-fix-error-on-interactive-command-g.patch
  + 0017-tools-kvm_stat-add-new-command-line-switch-i.patch
  + 0018-tools-kvm_stat-add-new-interactive-command-b.patch
  + 0019-tools-kvm_stat-add-f-help-to-get-the-available-event.patch
  + 0020-tools-kvm_stat-Add-Python-3-support-to-kvm_stat.patch
  + 0021-tools-kvm_stat-fix-command-line-option-g.patch
  + 0022-tools-kvm_stat-fix-drilldown-in-events-by-guests-mod.patch
  + 0023-tools-kvm_stat-fix-missing-field-update-after-filter.patch
  + 0024-tools-kvm_stat-fix-extra-handling-of-help-with-field.patch
  + 0025-tools-kvm_stat-add-hint-on-f-help-to-man-page.patch
  + 0026-tools-kvm_stat-fix-child-trace-events-accounting.patch
  + 0027-tools-kvm_stat-handle-invalid-regular-expressions.patch
  + 0028-tools-kvm_stat-suppress-usage-information-on-command.patch
  + 0029-tools-kvm_stat-stop-ignoring-unhandled-arguments.patch
  + 0030-tools-kvm_stat-add-line-for-totals.patch
  + 0031-tools-kvm_stat-sort-f-help-output.patch
  + 0032-tools-kvm_stat-simplify-the-sortkey-function.patch
  + 0033-tools-kvm_stat-use-a-namedtuple-for-storing-the-valu.patch
  + 0034-tools-kvm_stat-use-a-more-pythonic-way-to-iterate-ov.patch
  + 0035-tools-kvm_stat-avoid-is-for-equality-checks.patch
  + 0036-tools-kvm_stat-fix-crash-when-filtering-out-all-non-.patch
  + 0037-tools-kvm_stat-print-error-on-invalid-regex.patch
  + 0038-tools-kvm_stat-fix-debugfs-handling.patch
  + 0039-tools-kvm_stat-mark-private-methods-as-such.patch
  + 0040-tools-kvm_stat-eliminate-extra-guest-pid-selection-d.patch
  + 0041-tools-kvm_stat-separate-drilldown-and-fields-filteri.patch
  + 0042-tools-kvm_stat-group-child-events-indented-after-par.patch
  + 0043-tools-kvm_stat-print-Total-line-for-multiple-events-.patch
* Fri Nov  9 2018 Bruce Rogers <brogers@suse.com>
- Add "Requires: python-curses" in kvm_stat.spec as this package
  needs to be installed to make kvm_stat work. (bsc#1114465)
* Thu Jul 26 2018 brogers@suse.com
- Add a qemu package conflicts directive since SLE releases provided
  the kvm_stat script via the qemu package, while older openSUSE
  releases had provided it through qemu-kvm, which is already
  handled. (bsc#1102746 related)
- Change package to be noarch. kvm_stat is written in python, and
  is useful to multiple architectures which have kvm support.
- Drop the kernel README and CREDITS files, since they don't really
  represent this package in any meaningful way.
- Minor spec file tweaks performed.
* Thu Dec  7 2017 dimstar@opensuse.org
- Escape the usage of %%{VERSION} when calling out to rpm.
  RPM 4.14 has %%{VERSION} defined as 'the main packages version'.
* Sun Oct 30 2016 jengelh@inai.de
- Simplify copying around
* Mon Oct 17 2016 brogers@suse.com
- soften runtime Requires kernel version to a Recommends since it
  really doesn't fully enforce anything which is "required".
* Mon Oct 17 2016 brogers@suse.com
- fix conflicts line to be against qemu-kvm, not qemu.
* Wed Jun 22 2016 brogers@suse.com
- initial check-in of kvm_stat package. Previously the kvm_stat
  tool was included with the qemu sources, but because kvm_stat simply
  represents kvm module information, as of v4.7.0 of the linux kernel,
  kvm_stat was included in the tools directory under the linux source,
  hence this package was born.
